@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2013-2018 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -15,10 +15,13 @@
 import asyncio
 
 from .constants import DEFAULT_LANG, DISC_PROTOCOL_ERROR
+
 from .gss import GSSError
-from .logging import logger
-from .misc import DisconnectError, PasswordChangeRequired
-from .packet import Boolean, Byte, String, UInt32, SSHPacketHandler
+
+from .misc import DisconnectError, PasswordChangeRequired, get_symbol_names
+
+from .packet import Boolean, String, UInt32, SSHPacketHandler
+
 from .saslprep import saslprep, SASLPrepError
 
 
@@ -35,13 +38,13 @@ MSG_USERAUTH_GSSAPI_MIC               = 66
 # SSH message values for public key auth
 MSG_USERAUTH_PK_OK                    = 60
 
-# SSH message values for password auth
-MSG_USERAUTH_PASSWD_CHANGEREQ         = 60
-
-# SSH message values for 'keyboard-interactive' auth
+# SSH message values for keyboard-interactive auth
 MSG_USERAUTH_INFO_REQUEST             = 60
 MSG_USERAUTH_INFO_RESPONSE            = 61
 
+# SSH message values for password auth
+MSG_USERAUTH_PASSWD_CHANGEREQ         = 60
+
 # pylint: enable=bad-whitespace
 
 _auth_methods = []
@@ -54,8 +57,20 @@ class _Auth(SSHPacketHandler):
 
     def __init__(self, conn, coro):
         self._conn = conn
+        self._logger = conn.logger
         self._coro = conn.create_task(coro)
 
+    def send_packet(self, pkttype, *args):
+        """Send an auth packet"""
+
+        self._conn.send_packet(pkttype, *args, handler=self)
+
+    @property
+    def logger(self):
+        """A logger associated with this authentication handler"""
+
+        return self._logger
+
     def create_task(self, coro):
         """Create an asynchronous auth task"""
 
@@ -117,6 +132,8 @@ class _ClientGSSKexAuth(_ClientAuth):
         """Start client GSS key exchange authentication"""
 
         if self._conn.gss_kex_auth_requested():
+            self.logger.debug1('Trying GSS key exchange auth')
+
             yield from self.send_request(key=self._conn.get_gss_context())
         else:
             self._conn.try_next_auth()
@@ -125,6 +142,8 @@ class _ClientGSSKexAuth(_ClientAuth):
 class _ClientGSSMICAuth(_ClientAuth):
     """Client side implementation of GSS MIC auth"""
 
+    _handler_names = get_symbol_names(globals(), 'MSG_USERAUTH_GSSAPI_')
+
     def __init__(self, conn, method):
         super().__init__(conn, method)
 
@@ -136,6 +155,8 @@ class _ClientGSSMICAuth(_ClientAuth):
         """Start client GSS MIC authentication"""
 
         if self._conn.gss_mic_auth_requested():
+            self.logger.debug1('Trying GSS MIC auth')
+
             self._gss = self._conn.get_gss_context()
             mechs = b''.join((String(mech) for mech in self._gss.mechs))
             yield from self.send_request(UInt32(len(self._gss.mechs)), mechs)
@@ -148,12 +169,12 @@ class _ClientGSSMICAuth(_ClientAuth):
         if self._gss.provides_integrity:
             data = self._conn.get_userauth_request_data(self._method)
 
-            self._conn.send_packet(Byte(MSG_USERAUTH_GSSAPI_MIC),
-                                   String(self._gss.sign(data)))
+            self.send_packet(MSG_USERAUTH_GSSAPI_MIC,
+                             String(self._gss.sign(data)))
         else:
-            self._conn.send_packet(Byte(MSG_USERAUTH_GSSAPI_EXCHANGE_COMPLETE))
+            self.send_packet(MSG_USERAUTH_GSSAPI_EXCHANGE_COMPLETE)
 
-    def _process_response(self, pkttype, packet):
+    def _process_response(self, pkttype, pktid, packet):
         """Process a GSS response from the server"""
 
         # pylint: disable=unused-argument
@@ -167,21 +188,19 @@ class _ClientGSSMICAuth(_ClientAuth):
         try:
             token = self._gss.step()
 
-            self._conn.send_packet(Byte(MSG_USERAUTH_GSSAPI_TOKEN),
-                                   String(token))
+            self.send_packet(MSG_USERAUTH_GSSAPI_TOKEN, String(token))
 
             if self._gss.complete:
                 self._finish()
         except GSSError as exc:
             if exc.token:
-                self._conn.send_packet(Byte(MSG_USERAUTH_GSSAPI_ERRTOK),
-                                       String(exc.token))
+                self.send_packet(MSG_USERAUTH_GSSAPI_ERRTOK, String(exc.token))
 
             self._conn.try_next_auth()
 
         return True
 
-    def _process_token(self, pkttype, packet):
+    def _process_token(self, pkttype, pktid, packet):
         """Process a GSS token from the server"""
 
         # pylint: disable=unused-argument
@@ -193,21 +212,19 @@ class _ClientGSSMICAuth(_ClientAuth):
             token = self._gss.step(token)
 
             if token:
-                self._conn.send_packet(Byte(MSG_USERAUTH_GSSAPI_TOKEN),
-                                       String(token))
+                self.send_packet(MSG_USERAUTH_GSSAPI_TOKEN, String(token))
 
             if self._gss.complete:
                 self._finish()
         except GSSError as exc:
             if exc.token:
-                self._conn.send_packet(Byte(MSG_USERAUTH_GSSAPI_ERRTOK),
-                                       String(exc.token))
+                self.send_packet(MSG_USERAUTH_GSSAPI_ERRTOK, String(exc.token))
 
             self._conn.try_next_auth()
 
         return True
 
-    def _process_error(self, pkttype, packet):
+    def _process_error(self, pkttype, pktid, packet):
         """Process a GSS error from the server"""
 
         # pylint: disable=unused-argument
@@ -218,13 +235,12 @@ class _ClientGSSMICAuth(_ClientAuth):
         _ = packet.get_string()         # lang
         packet.check_end()
 
-        logger.warning('GSS error from server: %s',
-                       msg.decode('utf-8', errors='ignore'))
+        self.logger.warning('GSS error from server: %s', msg)
         self._got_error = True
 
         return True
 
-    def _process_error_token(self, pkttype, packet):
+    def _process_error_token(self, pkttype, pktid, packet):
         """Process a GSS error token from the server"""
 
         # pylint: disable=no-self-use,unused-argument
@@ -236,25 +252,23 @@ class _ClientGSSMICAuth(_ClientAuth):
             self._gss.step(token)
         except GSSError as exc:
             if not self._got_error: # pragma: no cover
-                logger.warning('GSS error from server: %s', str(exc))
+                self.logger.warning('GSS error from server: %s', str(exc))
 
         return True
 
-    # pylint: disable=bad-whitespace
-
-    packet_handlers = {
+    _packet_handlers = {
         MSG_USERAUTH_GSSAPI_RESPONSE: _process_response,
         MSG_USERAUTH_GSSAPI_TOKEN:    _process_token,
         MSG_USERAUTH_GSSAPI_ERROR:    _process_error,
         MSG_USERAUTH_GSSAPI_ERRTOK:   _process_error_token
     }
 
-    # pylint: enable=bad-whitespace
-
 
 class _ClientPublicKeyAuth(_ClientAuth):
     """Client side implementation of public key auth"""
 
+    _handler_names = get_symbol_names(globals(), 'MSG_USERAUTH_PK_')
+
     @asyncio.coroutine
     def _start(self):
         """Start client public key authentication"""
@@ -265,6 +279,9 @@ class _ClientPublicKeyAuth(_ClientAuth):
             self._conn.try_next_auth()
             return
 
+        self.logger.debug1('Trying public key auth with %s key',
+                           self._keypair.algorithm)
+
         yield from self.send_request(Boolean(False),
                                      String(self._keypair.algorithm),
                                      String(self._keypair.public_data))
@@ -273,12 +290,15 @@ class _ClientPublicKeyAuth(_ClientAuth):
     def _send_signed_request(self):
         """Send signed public key request"""
 
+        self.logger.debug1('Signing request with %s key',
+                           self._keypair.algorithm)
+
         yield from self.send_request(Boolean(True),
                                      String(self._keypair.algorithm),
                                      String(self._keypair.public_data),
                                      key=self._keypair)
 
-    def _process_public_key_ok(self, pkttype, packet):
+    def _process_public_key_ok(self, pkttype, pktid, packet):
         """Process a public key ok response"""
 
         # pylint: disable=unused-argument
@@ -294,7 +314,7 @@ class _ClientPublicKeyAuth(_ClientAuth):
         self.create_task(self._send_signed_request())
         return True
 
-    packet_handlers = {
+    _packet_handlers = {
         MSG_USERAUTH_PK_OK: _process_public_key_ok
     }
 
@@ -302,6 +322,8 @@ class _ClientPublicKeyAuth(_ClientAuth):
 class _ClientKbdIntAuth(_ClientAuth):
     """Client side implementation of keyboard-interactive auth"""
 
+    _handler_names = get_symbol_names(globals(), 'MSG_USERAUTH_INFO_')
+
     @asyncio.coroutine
     def _start(self):
         """Start client keyboard interactive authentication"""
@@ -312,6 +334,8 @@ class _ClientKbdIntAuth(_ClientAuth):
             self._conn.try_next_auth()
             return
 
+        self.logger.debug1('Trying keyboard-interactive auth')
+
         yield from self.send_request(String(''), String(submethods))
 
     @asyncio.coroutine
@@ -326,11 +350,10 @@ class _ClientKbdIntAuth(_ClientAuth):
             self._conn.try_next_auth()
             return
 
-        self._conn.send_packet(Byte(MSG_USERAUTH_INFO_RESPONSE),
-                               UInt32(len(responses)),
-                               b''.join(String(r) for r in responses))
+        self.send_packet(MSG_USERAUTH_INFO_RESPONSE, UInt32(len(responses)),
+                         b''.join(String(r) for r in responses))
 
-    def _process_info_request(self, pkttype, packet):
+    def _process_info_request(self, pkttype, pktid, packet):
         """Process a keyboard interactive authentication request"""
 
         # pylint: disable=unused-argument
@@ -366,7 +389,7 @@ class _ClientKbdIntAuth(_ClientAuth):
 
         return True
 
-    packet_handlers = {
+    _packet_handlers = {
         MSG_USERAUTH_INFO_REQUEST: _process_info_request
     }
 
@@ -374,6 +397,8 @@ class _ClientKbdIntAuth(_ClientAuth):
 class _ClientPasswordAuth(_ClientAuth):
     """Client side implementation of password auth"""
 
+    _handler_names = get_symbol_names(globals(), 'MSG_USERAUTH_PASSWD_')
+
     def __init__(self, conn, method):
         super().__init__(conn, method)
 
@@ -389,6 +414,8 @@ class _ClientPasswordAuth(_ClientAuth):
             self._conn.try_next_auth()
             return
 
+        self.logger.debug1('Trying password auth')
+
         yield from self.send_request(Boolean(False), String(password))
 
     @asyncio.coroutine
@@ -402,6 +429,8 @@ class _ClientPasswordAuth(_ClientAuth):
             self._conn.try_next_auth()
             return
 
+        self.logger.debug1('Trying to chsnge password')
+
         old_password, new_password = result
 
         self._password_change = True
@@ -420,7 +449,7 @@ class _ClientPasswordAuth(_ClientAuth):
             self._password_change = False
             self._conn.password_change_failed()
 
-    def _process_password_change(self, pkttype, packet):
+    def _process_password_change(self, pkttype, pktid, packet):
         """Process a password change request"""
 
         # pylint: disable=unused-argument
@@ -440,7 +469,7 @@ class _ClientPasswordAuth(_ClientAuth):
 
         return True
 
-    packet_handlers = {
+    _packet_handlers = {
         MSG_USERAUTH_PASSWD_CHANGEREQ: _process_password_change
     }
 
@@ -508,6 +537,8 @@ class _ServerGSSKexAuth(_ServerAuth):
         mic = packet.get_string()
         packet.check_end()
 
+        self.logger.debug1('Trying GSS key exchange auth')
+
         data = self._conn.get_userauth_request_data(self._method)
 
         if (self._gss.complete and self._gss.verify(data, mic) and
@@ -522,6 +553,8 @@ class _ServerGSSKexAuth(_ServerAuth):
 class _ServerGSSMICAuth(_ServerAuth):
     """Server side implementation of GSS MIC auth"""
 
+    _handler_names = get_symbol_names(globals(), 'MSG_USERAUTH_GSSAPI_')
+
     def __init__(self, conn, username, method, packet):
         super().__init__(conn, username, method, packet)
 
@@ -555,8 +588,9 @@ class _ServerGSSMICAuth(_ServerAuth):
             self.send_failure()
             return
 
-        self._conn.send_packet(Byte(MSG_USERAUTH_GSSAPI_RESPONSE),
-                               String(match))
+        self.logger.debug1('Trying GSS MIC auth')
+
+        self.send_packet(MSG_USERAUTH_GSSAPI_RESPONSE, String(match))
 
     @asyncio.coroutine
     def _finish(self):
@@ -569,7 +603,7 @@ class _ServerGSSMICAuth(_ServerAuth):
         else:
             self.send_failure()
 
-    def _process_token(self, pkttype, packet):
+    def _process_token(self, pkttype, pktid, packet):
         """Process a GSS token from the client"""
 
         # pylint: disable=unused-argument
@@ -581,22 +615,20 @@ class _ServerGSSMICAuth(_ServerAuth):
             token = self._gss.step(token)
 
             if token:
-                self._conn.send_packet(Byte(MSG_USERAUTH_GSSAPI_TOKEN),
-                                       String(token))
+                self.send_packet(MSG_USERAUTH_GSSAPI_TOKEN, String(token))
         except GSSError as exc:
-            self._conn.send_packet(Byte(MSG_USERAUTH_GSSAPI_ERROR),
-                                   UInt32(exc.maj_code), UInt32(exc.min_code),
-                                   String(str(exc)), String(DEFAULT_LANG))
+            self.send_packet(MSG_USERAUTH_GSSAPI_ERROR, UInt32(exc.maj_code),
+                             UInt32(exc.min_code), String(str(exc)),
+                             String(DEFAULT_LANG))
 
             if exc.token:
-                self._conn.send_packet(Byte(MSG_USERAUTH_GSSAPI_ERRTOK),
-                                       String(exc.token))
+                self.send_packet(MSG_USERAUTH_GSSAPI_ERRTOK, String(exc.token))
 
             self.send_failure()
 
         return True
 
-    def _process_exchange_complete(self, pkttype, packet):
+    def _process_exchange_complete(self, pkttype, pktid, packet):
         """Process a GSS exchange complete message from the client"""
 
         # pylint: disable=unused-argument
@@ -610,7 +642,7 @@ class _ServerGSSMICAuth(_ServerAuth):
 
         return True
 
-    def _process_error_token(self, pkttype, packet):
+    def _process_error_token(self, pkttype, pktid, packet):
         """Process a GSS error token from the client"""
 
         # pylint: disable=unused-argument
@@ -621,11 +653,11 @@ class _ServerGSSMICAuth(_ServerAuth):
         try:
             self._gss.step(token)
         except GSSError as exc:
-            logger.warning('GSS error from client: %s', str(exc))
+            self.logger.warning('GSS error from client: %s', str(exc))
 
         return True
 
-    def _process_mic(self, pkttype, packet):
+    def _process_mic(self, pkttype, pktid, packet):
         """Process a GSS MIC from the client"""
 
         # pylint: disable=unused-argument
@@ -643,17 +675,13 @@ class _ServerGSSMICAuth(_ServerAuth):
 
         return True
 
-    # pylint: disable=bad-whitespace
-
-    packet_handlers = {
+    _packet_handlers = {
         MSG_USERAUTH_GSSAPI_TOKEN:             _process_token,
         MSG_USERAUTH_GSSAPI_EXCHANGE_COMPLETE: _process_exchange_complete,
         MSG_USERAUTH_GSSAPI_ERRTOK:            _process_error_token,
         MSG_USERAUTH_GSSAPI_MIC:               _process_mic
     }
 
-    # pylint: enable=bad-whitespace
-
 
 class _ServerPublicKeyAuth(_ServerAuth):
     """Server side implementation of public key auth"""
@@ -681,13 +709,18 @@ class _ServerPublicKeyAuth(_ServerAuth):
 
         packet.check_end()
 
+        if sig_present:
+            self.logger.debug1('Verifying request with %s key', algorithm)
+        else:
+            self.logger.debug1('Trying public key auth with %s key', algorithm)
+
         if (yield from self._conn.validate_public_key(self._username, key_data,
                                                       msg, signature)):
             if sig_present:
                 self.send_success()
             else:
-                self._conn.send_packet(Byte(MSG_USERAUTH_PK_OK),
-                                       String(algorithm), String(key_data))
+                self.send_packet(MSG_USERAUTH_PK_OK, String(algorithm),
+                                 String(key_data))
         else:
             self.send_failure()
 
@@ -695,6 +728,8 @@ class _ServerPublicKeyAuth(_ServerAuth):
 class _ServerKbdIntAuth(_ServerAuth):
     """Server side implementation of keyboard-interactive auth"""
 
+    _handler_names = get_symbol_names(globals(), 'MSG_USERAUTH_INFO_')
+
     @classmethod
     def supported(cls, conn):
         """Return whether keyboard interactive authentication is supported"""
@@ -716,6 +751,8 @@ class _ServerKbdIntAuth(_ServerAuth):
             raise DisconnectError(DISC_PROTOCOL_ERROR, 'Invalid keyboard '
                                   'interactive auth request') from None
 
+        self.logger.debug1('Trying keyboard-interactive auth')
+
         challenge = yield from self._conn.get_kbdint_challenge(self._username,
                                                                lang,
                                                                submethods)
@@ -731,10 +768,9 @@ class _ServerKbdIntAuth(_ServerAuth):
             prompts = (String(prompt) + Boolean(echo)
                        for prompt, echo in prompts)
 
-            self._conn.send_packet(Byte(MSG_USERAUTH_INFO_REQUEST),
-                                   String(name), String(instruction),
-                                   String(lang), UInt32(num_prompts),
-                                   *prompts)
+            self.send_packet(MSG_USERAUTH_INFO_REQUEST, String(name),
+                             String(instruction), String(lang),
+                             UInt32(num_prompts), *prompts)
         elif challenge:
             self.send_success()
         else:
@@ -749,7 +785,7 @@ class _ServerKbdIntAuth(_ServerAuth):
                                                            responses)
         self._send_challenge(next_challenge)
 
-    def _process_info_response(self, pkttype, packet):
+    def _process_info_response(self, pkttype, pktid, packet):
         """Process a keyboard interactive authentication response"""
 
         # pylint: disable=unused-argument
@@ -772,7 +808,7 @@ class _ServerKbdIntAuth(_ServerAuth):
         self.create_task(self._validate_response(responses))
         return True
 
-    packet_handlers = {
+    _packet_handlers = {
         MSG_USERAUTH_INFO_RESPONSE: _process_info_response
     }
 
@@ -804,10 +840,14 @@ class _ServerPasswordAuth(_ServerAuth):
 
         try:
             if password_change:
+                self.logger.debug1('Trying to chsnge password')
+
                 result = yield from self._conn.change_password(self._username,
                                                                password,
                                                                new_password)
             else:
+                self.logger.debug1('Trying password auth')
+
                 result = \
                     yield from self._conn.validate_password(self._username,
                                                             password)
@@ -817,8 +857,8 @@ class _ServerPasswordAuth(_ServerAuth):
             else:
                 self.send_failure()
         except PasswordChangeRequired as exc:
-            self._conn.send_packet(Byte(MSG_USERAUTH_PASSWD_CHANGEREQ),
-                                   String(exc.prompt), String(exc.lang))
+            self.send_packet(MSG_USERAUTH_PASSWD_CHANGEREQ,
+                             String(exc.prompt), String(exc.lang))
 
 
 def register_auth_method(alg, client_handler, server_handler):
