@@ -275,8 +275,9 @@ def import_authorized_keys(data):
        This function imports public keys and associated options in
        OpenSSH authorized keys format.
 
-       :param str data:
+       :param data:
            The key data to import.
+       :type data: `str`
 
        :returns: An :class:`SSHAuthorizedKeys` object
 
@@ -291,8 +292,9 @@ def read_authorized_keys(filename):
        This function reads public keys and associated options in
        OpenSSH authorized_keys format from a file.
 
-       :param str filename:
+       :param filename:
            The file to read the keys from.
+       :type filename: `str`
 
        :returns: An :class:`SSHAuthorizedKeys` object
 
