@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2015 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2014-2017 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -15,27 +15,34 @@
 from cryptography.exceptions import InvalidSignature
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives.asymmetric.padding import PKCS1v15
-from cryptography.hazmat.primitives.hashes import SHA1
+from cryptography.hazmat.primitives.hashes import SHA1, SHA256, SHA512
 from cryptography.hazmat.primitives.asymmetric import rsa
 
+from .misc import PyCAKey
+
 # Short variable names are used here, matching names in the spec
 # pylint: disable=invalid-name
 
 
-class _RSAKey:
-    """Base class for shum around PyCA for RSA keys"""
+class _RSAKey(PyCAKey):
+    """Base class for shim around PyCA for RSA keys"""
 
-    def __init__(self, n, e, d=None, p=None, q=None,
-                 dmp1=None, dmq1=None, iqmp=None):
-        self._pub = rsa.RSAPublicNumbers(e, n)
+    def __init__(self, pyca_key, pub, priv=None):
+        super().__init__(pyca_key)
 
-        if d:
-            self._priv = rsa.RSAPrivateNumbers(p, q, d, dmp1, dmq1,
-                                               iqmp, self._pub)
-            self._priv_key = self._priv.private_key(default_backend())
+        self._pub = pub
+        self._priv = priv
+
+    @staticmethod
+    def get_hash(algorithm):
+        """Return hash algorithm to use for signature"""
+
+        if algorithm == b'rsa-sha2-512':
+            return SHA512()
+        elif algorithm in (b'rsa-sha2-256', b'rsa2048-sha256'):
+            return SHA256()
         else:
-            self._priv = None
-            self._pub_key = self._pub.public_key(default_backend())
+            return SHA1()
 
     @property
     def n(self):
@@ -89,25 +96,51 @@ class _RSAKey:
 class RSAPrivateKey(_RSAKey):
     """A shim around PyCA for RSA private keys"""
 
-    def sign(self, data):
+    @classmethod
+    def construct(cls, n, e, d, p, q, dmp1, dmq1, iqmp):
+        """Construct an RSA private key"""
+
+        pub = rsa.RSAPublicNumbers(e, n)
+        priv = rsa.RSAPrivateNumbers(p, q, d, dmp1, dmq1, iqmp, pub)
+        priv_key = priv.private_key(default_backend())
+
+        return cls(priv_key, pub, priv)
+
+    @classmethod
+    def generate(cls, key_size, exponent):
+        """Generate a new RSA private key"""
+
+        priv_key = rsa.generate_private_key(exponent, key_size,
+                                            default_backend())
+        priv = priv_key.private_numbers()
+        pub = priv.public_numbers
+
+        return cls(priv_key, pub, priv)
+
+    def sign(self, data, algorithm):
         """Sign a block of data"""
 
-        signer = self._priv_key.signer(PKCS1v15(), SHA1())
-        signer.update(data)
-        return signer.finalize()
+        return self.pyca_key.sign(data, PKCS1v15(), self.get_hash(algorithm))
 
 
 class RSAPublicKey(_RSAKey):
     """A shim around PyCA for RSA public keys"""
 
-    def verify(self, data, sig):
+    @classmethod
+    def construct(cls, n, e):
+        """Construct an RSA public key"""
+
+        pub = rsa.RSAPublicNumbers(e, n)
+        pub_key = pub.public_key(default_backend())
+
+        return cls(pub_key, pub)
+
+    def verify(self, data, sig, algorithm):
         """Verify the signature on a block of data"""
 
-        verifier = self._pub_key.verifier(sig, PKCS1v15(), SHA1())
-        verifier.update(data)
-
         try:
-            verifier.verify()
+            self.pyca_key.verify(sig, data, PKCS1v15(),
+                                 self.get_hash(algorithm))
             return True
         except InvalidSignature:
             return False
