@@ -456,16 +456,17 @@ class SSHLineEditor:
         self._end_column = self._cursor
         self._draw_input()
 
+
 class SSHLineEditorChannel:
     """Input line editor channel wrapper
 
-       When creating server channels with ``line_editor`` set to ``True``,
+       When creating server channels with `line_editor` set to `True`,
        this class is wrapped around the channel, providing the caller with
        the ability to enable and disable input line editing and echoing.
 
        .. note:: Line editing is only available when a psuedo-terminal
                  is requested on the server channel and the character
-                 encoding on the channel is not set to ``None``.
+                 encoding on the channel is not set to `None`.
 
     """
 
@@ -497,11 +498,15 @@ class SSHLineEditorChannel:
            only full lines of input are sent to the session, and each
            line of input can be edited before it is sent.
 
-           :param bool line_mode:
+           :param line_mode:
                Whether or not to process input a line at a time
+           :type line_mode: `bool`
 
         """
 
+        self._orig_chan.logger.info('%s line editor',
+                                    'Enabling' if line_mode else 'Disabling')
+
         self._editor.set_line_mode(line_mode)
 
     def set_echo(self, echo):
@@ -510,11 +515,15 @@ class SSHLineEditorChannel:
            This method enables or disables echoing of input data when
            input line editing is enabled.
 
-           :param bool echo:
+           :param echo:
                Whether or not input to echo input as it is entered
+           :type echo: `bool`
 
         """
 
+        self._orig_chan.logger.info('%s echo',
+                                    'Enabling' if echo else 'Disabling')
+
         self._editor.set_echo(echo)
 
     def write(self, data, datatype=None):
