@@ -26,7 +26,7 @@ class SSHListener(asyncio.AbstractServer):
         """Return the port number being listened on
 
            This method returns the port number that the remote listener
-           was bound to. When the requested remote listening port is ``0``
+           was bound to. When the requested remote listening port is `0`
            to indicate a dynamic port, this method can be called to
            determine what listening port was selected. This function
            only applies to TCP listeners.
@@ -245,6 +245,7 @@ def create_tcp_forward_listener(conn, loop, coro, listen_host, listen_port):
 
         if listen_port == 0:
             listen_port = sock.getsockname()[1]
+            conn.logger.debug1('Assigning dynamic port %d', listen_port)
 
         server = yield from loop.create_server(protocol_factory, sock=sock)
         servers.append(server)
