@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2013-2018 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -13,6 +13,7 @@
 """Miscellaneous utility classes and functions"""
 
 import asyncio
+import codecs
 import functools
 import ipaddress
 import os
@@ -43,7 +44,26 @@ randrange = _random.randrange
 if python344:
     create_task = asyncio.ensure_future
 else: # pragma: no cover
-    create_task = asyncio.async    # pylint: disable=no-member
+    create_task = getattr(asyncio, 'async')
+
+
+def hide_empty(value, prefix=', '):
+    """Return a string with optional prefix if value is non-empty"""
+
+    value = str(value)
+    return prefix + value if value else ''
+
+
+def plural(length, label, suffix='s'):
+    """Return a label with an optional plural suffix"""
+
+    return '%d %s%s' % (length, label, suffix if length != 1 else '')
+
+
+def to_hex(data):
+    """Convert binary data to a hex string"""
+
+    return codecs.encode(data, 'hex')
 
 
 def all_ints(seq):
@@ -52,6 +72,13 @@ def all_ints(seq):
     return all(isinstance(i, int) for i in seq)
 
 
+def get_symbol_names(symbols, prefix, strip_leading=0):
+    """Return a mapping from values to symbol names for logging"""
+
+    return {value: name[strip_leading:] for name, value in symbols.items()
+            if name.startswith(prefix)}
+
+
 # Default file names in .ssh directory to read private keys from
 _DEFAULT_KEY_FILES = ('id_ed25519', 'id_ecdsa', 'id_rsa', 'id_dsa')
 
@@ -200,6 +227,19 @@ class Record:
                            ', '.join('%s=%r' % (k, getattr(self, k))
                                      for k in self.__slots__))
 
+    def __str__(self):
+        values = ((k, self._format(k, getattr(self, k)))
+                  for k in self.__slots__)
+
+        return ', '.join('%s: %s' % (k, v) for k, v in values if v is not None)
+
+    def _format(self, k, v):
+        """Format a field as a string"""
+
+        # pylint: disable=no-self-use,unused-argument
+
+        return str(v)
+
 
 class Error(Exception):
     """General SSH error"""
@@ -218,13 +258,16 @@ class DisconnectError(Error):
        the SSH connection to be disconnected. Exception codes should be
        taken from :ref:`disconnect reason codes <DisconnectReasons>`.
 
-       :param int code:
+       :param code:
            Disconnect reason, taken from :ref:`disconnect reason
            codes <DisconnectReasons>`
-       :param str reason:
+       :param reason:
            A human-readable reason for the disconnect
-       :param str lang:
+       :param lang:
            The language the reason is in
+       :type code: `int`
+       :type reason: `str`
+       :type lang: `str`
 
     """
 
@@ -238,13 +281,16 @@ class ChannelOpenError(Error):
        This exception is raised by connection handlers to report
        channel open failures.
 
-       :param int code:
+       :param code:
            Channel open failure  reason, taken from :ref:`channel open
            failure reason codes <ChannelOpenFailureReasons>`
-       :param str reason:
+       :param reason:
            A human-readable reason for the channel open failure
-       :param str lang:
+       :param lang:
            The language the reason is in
+       :type code: `int`
+       :type reason: `str`
+       :type lang: `str`
 
     """
 
@@ -260,10 +306,12 @@ class PasswordChangeRequired(Exception):
        shouuld be raised when the password provided is valid but
        expired, to trigger the client to provide a new password.
 
-       :param str prompt:
+       :param prompt:
            The prompt requesting that the user enter a new password
-       :param str lang:
+       :param lang:
            The language that the prompt is in
+       :type prompt: `str`
+       :type lang: `str`
 
     """
 
@@ -279,8 +327,9 @@ class BreakReceived(Exception):
        This exception is raised on an SSH server stdin stream when the
        client sends a break on the channel.
 
-       :param int msec:
+       :param msec:
            The duration of the break in milliseconds
+       :type msec: `int`
 
     """
 
@@ -295,8 +344,9 @@ class SignalReceived(Exception):
        This exception is raised on an SSH server stdin stream when the
        client sends a signal on the channel.
 
-       :param str signal:
+       :param signal:
            The name of the signal sent by the client
+       :type signal: `str`
 
     """
 
@@ -311,14 +361,18 @@ class TerminalSizeChanged(Exception):
        This exception is raised on an SSH server stdin stream when the
        client sends a terminal size change on the channel.
 
-       :param int width:
+       :param width:
            The new terminal width
-       :param int height:
+       :param height:
            The new terminal height
-       :param int pixwidth:
+       :param pixwidth:
            The new terminal width in pixels
-       :param int pixheight:
+       :param pixheight:
            The new terminal height in pixels
+       :type width: `int`
+       :type height: `int`
+       :type pixwidth: `int`
+       :type pixheight: `int`
 
     """
 
