@@ -23,6 +23,8 @@ import stat
 from .constants import DEFAULT_LANG
 from .constants import FX_BAD_MESSAGE, FX_CONNECTION_LOST, FX_FAILURE
 
+from .misc import plural
+
 from .sftp import LocalFile, match_glob
 from .sftp import SFTP_BLOCK_SIZE, SFTPAttrs, SFTPError, SFTPServerFile
 
@@ -41,7 +43,7 @@ def _parse_t_args(args):
     """Parse argument to an SCP time request"""
 
     try:
-        atime, _, mtime, _ = args.split()
+        mtime, _, atime, _ = args.split()
         return int(atime), int(mtime)
     except ValueError:
         raise SCPError(FX_BAD_MESSAGE, 'Invalid time request') from None
@@ -89,6 +91,9 @@ def _start_remote(conn, source, must_be_dir, preserve, recurse, path):
                (b'-p ' if preserve else b'') +
                (b'-r ' if recurse else b'') + path)
 
+    conn.logger.get_child('sftp').info('Starting remote SCP, args: %s',
+                                       command[4:])
+
     writer, reader, _ = yield from conn.open_session(command, encoding=None)
 
     return reader, writer
@@ -147,6 +152,14 @@ class _SCPHandler:
         self._writer = writer
         self._error_handler = error_handler
 
+        self._logger = reader.logger.get_child('sftp')
+
+    @property
+    def logger(self):
+        """A logger associated with this SCP handler"""
+
+        return self._logger
+
     @asyncio.coroutine
     def await_response(self):
         """Wait for an SCP response"""
@@ -166,12 +179,18 @@ class _SCPHandler:
             return SCPError(FX_FAILURE, reason[:-1], fatal=result != b'\x01',
                             suppress_send=True)
 
+        self.logger.debug1('Received SCP OK')
+
         return None
 
     def send_request(self, *args):
         """Send an SCP request"""
 
-        self._writer.write(b''.join(args) + b'\n')
+        request = b''.join(args)
+
+        self.logger.debug1('Sending SCP request: %s', request)
+
+        self._writer.write(request + b'\n')
 
     @asyncio.coroutine
     def make_request(self, *args):
@@ -188,12 +207,16 @@ class _SCPHandler:
     def send_data(self, data):
         """Send SCP file data"""
 
+        self.logger.debug1('Sending %s', plural(len(data), 'SCP data byte'))
+
         self._writer.write(data)
         yield from self._writer.drain()
 
     def send_ok(self):
         """Send an SCP OK response"""
 
+        self.logger.debug1('Sending SCP OK')
+
         self._writer.write(b'\0')
 
     def send_error(self, exc):
@@ -214,6 +237,9 @@ class _SCPHandler:
 
         fatal = getattr(exc, 'fatal', False)
 
+        self.logger.debug1('Sending SCP %serror: %s',
+                           'fatal ' if fatal else '', reason)
+
         self._writer.write((b'\x02' if fatal else b'\x01') +
                            b'scp: ' + reason + b'\n')
 
@@ -226,14 +252,26 @@ class _SCPHandler:
         if not request:
             return None, None
 
-        return request[:1], request[1:-1]
+        action, args = request[:1], request[1:-1]
+
+        if action not in b'\x01\x02':
+            self.logger.debug1('Received SCP request: %s%s', action, args)
+        else:
+            self.logger.debug1('Received SCP %serror: %s',
+                               'fatal ' if action != b'\x01'  else '', args)
+
+        return action, args
 
 
     @asyncio.coroutine
     def recv_data(self, n):
         """Receive SCP file data"""
 
-        return (yield from self._reader.read(n))
+        data = (yield from self._reader.read(n))
+
+        self.logger.debug1('Received %s', plural(len(data), 'SCP data byte'))
+
+        return data
 
     def handle_error(self, exc):
         """Handle an SCP error"""
@@ -245,15 +283,20 @@ class _SCPHandler:
         if not getattr(exc, 'suppress_send', False):
             self.send_error(exc)
 
+        self.logger.debug1('Handling SCP error: %s', exc)
+
         if getattr(exc, 'fatal', False) or self._error_handler is None:
             raise exc from None
         elif self._error_handler:
             self._error_handler(exc)
 
+    @asyncio.coroutine
     def close(self):
         """Close an SCP session"""
 
+        self.logger.info('Stopping remote SCP')
         self._writer.close()
+        yield from self._writer.channel.wait_closed()
 
 
 class _SCPSource(_SCPHandler):
@@ -282,7 +325,10 @@ class _SCPSource(_SCPHandler):
     def _make_t_request(self, attrs):
         """Make an SCP time request"""
 
-        args = '%d 0 %d 0' % (attrs.atime, attrs.mtime)
+        self.logger.info('    Preserving attrs: %s',
+                         SFTPAttrs(atime=attrs.atime, mtime=attrs.mtime))
+
+        args = '%d 0 %d 0' % (attrs.mtime, attrs.atime)
         yield from self.make_request(b'T', args.encode('ascii'))
 
     @asyncio.coroutine
@@ -294,6 +340,8 @@ class _SCPSource(_SCPHandler):
         local_exc = None
         offset = 0
 
+        self.logger.info('  Sending file %s, size %d', srcpath, size)
+
         try:
             yield from self._make_cd_request(b'C', attrs, size, srcpath)
 
@@ -335,6 +383,8 @@ class _SCPSource(_SCPHandler):
     def _send_dir(self, srcpath, dstpath, attrs):
         """Send directory over SCP"""
 
+        self.logger.info('  Starting send of directory %s', srcpath)
+
         yield from self._make_cd_request(b'D', attrs, 0, srcpath)
 
         for name in (yield from self._fs.listdir(srcpath)):
@@ -346,6 +396,8 @@ class _SCPSource(_SCPHandler):
 
         yield from self.make_request(b'E')
 
+        self.logger.info('  Finished send of directory %s', srcpath)
+
     @asyncio.coroutine
     def _send_files(self, srcpath, dstpath):
         """Send files via SCP"""
@@ -383,7 +435,7 @@ class _SCPSource(_SCPHandler):
         except (OSError, SFTPError) as exc:
             self.handle_error(exc)
         finally:
-            self.close()
+            yield from self.close()
 
 
 class _SCPSink(_SCPHandler):
@@ -409,6 +461,8 @@ class _SCPSink(_SCPHandler):
         local_exc = None
         offset = 0
 
+        self.logger.info('  Receiving file %s, size %d', dstpath, size)
+
         try:
             self.send_ok()
 
@@ -453,6 +507,8 @@ class _SCPSink(_SCPHandler):
         if not self._recurse:
             raise SCPError(FX_BAD_MESSAGE, 'Directory received without recurse')
 
+        self.logger.info('  Starting receive of directory %s', dstpath)
+
         if (yield from self._fs.exists(dstpath)):
             if not (yield from self._fs.isdir(dstpath)):
                 raise SCPError(FX_FAILURE, 'Not a directory', dstpath)
@@ -461,6 +517,8 @@ class _SCPSink(_SCPHandler):
 
         yield from self._recv_files(srcpath, dstpath)
 
+        self.logger.info('  Finished receive of directory %s', dstpath)
+
     @asyncio.coroutine
     def _recv_files(self, srcpath, dstpath):
         """Receive files over SCP"""
@@ -504,6 +562,7 @@ class _SCPSink(_SCPHandler):
                                                        new_dstpath, size)
 
                         if self._preserve:
+                            self.logger.info('    Preserving attrs: %s', attrs)
                             yield from self._fs.setstat(new_dstpath, attrs)
                     finally:
                         attrs = SFTPAttrs()
@@ -528,7 +587,7 @@ class _SCPSink(_SCPHandler):
         except (OSError, SFTPError, ValueError) as exc:
             self.handle_error(exc)
         finally:
-            self.close()
+            yield from self.close()
 
 
 class _SCPCopier:
@@ -539,16 +598,25 @@ class _SCPCopier:
                  error_handler=None):
         self._source = _SCPHandler(src_reader, src_writer)
         self._sink = _SCPHandler(dst_reader, dst_writer)
+        self._logger = self._source.logger
         self._block_size = block_size
         self._progress_handler = progress_handler
         self._error_handler = error_handler
 
+    @property
+    def logger(self):
+        """A logger associated with this SCP handler"""
+
+        return self._logger
+
     def _handle_error(self, exc):
         """Handle an SCP error"""
 
         if isinstance(exc, BrokenPipeError):
             exc = SCPError(FX_CONNECTION_LOST, 'Connection lost', fatal=True)
 
+        self.logger.debug1('Handling SCP error: %s', exc)
+
         if self._error_handler and not getattr(exc, 'fatal', False):
             self._error_handler(exc)
         else:
@@ -576,6 +644,8 @@ class _SCPCopier:
     def _copy_file(self, path, size):
         """Copy a file from one remote SCP server to another"""
 
+        self.logger.info('  Copying file %s, size %d', path, size)
+
         offset = 0
 
         while offset < size:
@@ -601,57 +671,80 @@ class _SCPCopier:
             self._handle_error(exc)
 
     @asyncio.coroutine
-    def run(self):
-        """Start SCP remote-to-remote transfer"""
+    def _copy_files(self):
+        """Copy files from one SCP server to another"""
 
-        paths = []
+        exc = yield from self._forward_response(self._sink, self._source)
 
-        try:
-            exc = yield from self._forward_response(self._sink, self._source)
+        if exc:
+            self._handle_error(exc)
+
+        pathlist = []
+        attrlist = []
+        attrs = SFTPAttrs()
+
+        while True:
+            action, args = yield from self._source.recv_request()
+
+            if not action:
+                break
+
+            self._sink.send_request(action, args)
+
+            if action in b'\x01\x02':
+                exc = SCPError(FX_FAILURE, args, fatal=action != b'\x01')
+                self._handle_error(exc)
+                continue
+
+            exc = yield from self._forward_response(self._sink,
+                                                    self._source)
 
             if exc:
                 self._handle_error(exc)
+                continue
 
-            while True:
-                action, args = yield from self._source.recv_request()
-
-                if not action:
-                    break
-
-                self._sink.send_request(action, args)
-
-                if action in b'\x01\x02':
-                    exc = SCPError(FX_FAILURE, args, fatal=action != b'\x01')
-                    self._handle_error(exc)
-                    continue
-
-                exc = yield from self._forward_response(self._sink,
-                                                        self._source)
-
-                if exc:
-                    self._handle_error(exc)
-                    continue
-
-                if action in b'CD':
-                    _, size, name = _parse_cd_args(args)
+            if action in b'CD':
+                try:
+                    attrs.permissions, size, name = _parse_cd_args(args)
 
                     if action == b'C':
-                        path = b'/'.join(paths + [name])
+                        path = b'/'.join(pathlist + [name])
                         yield from self._copy_file(path, size)
+                        self.logger.info('    Preserving attrs: %s', attrs)
                     else:
-                        paths.append(name)
-                elif action == b'E':
-                    if paths:
-                        paths.pop()
-                    else:
-                        break
-                elif action != b'T':
-                    raise SCPError(FX_BAD_MESSAGE, 'Unknown SCP action')
+                        pathlist.append(name)
+                        attrlist.append(attrs)
+                        self.logger.info('  Starting copy of directory %s',
+                                         b'/'.join(pathlist))
+                finally:
+                    attrs = SFTPAttrs()
+            elif action == b'E':
+                if pathlist:
+                    self.logger.info('  Finished copy of directory %s',
+                                     b'/'.join(pathlist))
+
+                    pathlist.pop()
+                    attrs = attrlist.pop()
+
+                    self.logger.info('    Preserving attrs: %s', attrs)
+                else:
+                    break
+            elif action == b'T':
+                attrs.atime, attrs.mtime = _parse_t_args(args)
+            else:
+                raise SCPError(FX_BAD_MESSAGE, 'Unknown SCP action')
+
+    @asyncio.coroutine
+    def run(self):
+        """Start SCP remote-to-remote transfer"""
+
+        try:
+            yield from self._copy_files()
         except (OSError, SFTPError) as exc:
             self._handle_error(exc)
         finally:
-            self._source.close()
-            self._sink.close()
+            yield from self._source.close()
+            yield from self._sink.close()
 
 
 @asyncio.coroutine
@@ -661,16 +754,16 @@ def scp(srcpaths, dstpath=None, *, preserve=False, recurse=False,
 
        This function is a coroutine which copies one or more files or
        directories using the SCP protocol. Source and destination paths
-       can be string or bytes values to reference local files or can be
-       a tuple of the form ``(conn, path)`` where ``conn`` is an open
+       can be `str` or `bytes` values to reference local files or can be
+       a tuple of the form `(conn, path)` where `conn` is an open
        :class:`SSHClientConnection` to reference files and directories
        on a remote system.
 
-       For convenience, a host name or tuple of the form ``(host, port)``
+       For convenience, a host name or tuple of the form `(host, port)`
        can be provided in place of the :class:`SSHClientConnection` to
        request that a new SSH connection be opened to a host using
-       default connect arguments. A string or bytes value of the form
-       ``'host:path'`` may also be used in place of the ``(conn, path)``
+       default connect arguments. A `str` or `bytes` value of the form
+       `'host:path'` may also be used in place of the `(conn, path)`
        tuple to make a new connection to the requested host on the
        default SSH port.
 
@@ -692,7 +785,7 @@ def scp(srcpaths, dstpath=None, *, preserve=False, recurse=False,
        a path or the path provided is empty, files are copied into the
        default destination working directory.
 
-       If preserve is ``True``, the access and modification times and
+       If preserve is `True`, the access and modification times and
        permissions of the original files and directories are set on the
        copied files. However, do to the timing of when this information
        is sent, the preserved access time will be what was set on the
@@ -700,7 +793,7 @@ def scp(srcpaths, dstpath=None, *, preserve=False, recurse=False,
        source file will no longer match the destination after the
        transfer completes.
 
-       If recurse is ``True`` and the source path points at a directory,
+       If recurse is `True` and the source path points at a directory,
        the entire subtree under that directory is copied.
 
        Symbolic links found on the source will have the contents of their
@@ -717,14 +810,14 @@ def scp(srcpaths, dstpath=None, *, preserve=False, recurse=False,
        block of a file is successfully copied. The arguments passed to
        this handler will be the relative path of the file being copied,
        bytes copied so far, and total bytes in the file being copied. If
-       multiple source paths are provided or recurse is set to ``True``,
+       multiple source paths are provided or recurse is set to `True`,
        the progress_handler will be called consecutively on each file
        being copied.
 
        If error_handler is specified and an error occurs during the copy,
        this handler will be called with the exception instead of it being
        raised. This is intended to primarily be used when multiple source
-       paths are provided or when recurse is set to ``True``, to allow
+       paths are provided or when recurse is set to `True`, to allow
        error information to be collected without aborting the copy of the
        remaining files. The error handler can raise an exception if it
        wants the copy to completely stop. Otherwise, after an error, the
@@ -734,16 +827,21 @@ def scp(srcpaths, dstpath=None, *, preserve=False, recurse=False,
            The paths of the source files or directories to copy
        :param dstpath: (optional)
            The path of the destination file or directory to copy into
-       :param bool preserve: (optional)
+       :param preserve: (optional)
            Whether or not to preserve the original file attributes
-       :param bool recurse: (optional)
+       :param recurse: (optional)
            Whether or not to recursively copy directories
-       :param int block_size: (optional)
+       :param block_size: (optional)
            The block size to use for file reads and writes
-       :param callable progress_handler: (optional)
+       :param progress_handler: (optional)
            The function to call to report copy progress
-       :param callable error_handler: (optional)
+       :param error_handler: (optional)
            The function to call when an error occurs
+       :type preserve: `bool`
+       :type recurse: `bool`
+       :type block_size: `int`
+       :type progress_handler: `callable`
+       :type error_handler: `callable`
 
        :raises: | :exc:`OSError` if a local file I/O error occurs
                 | :exc:`SFTPError` if the server returns an error
@@ -813,10 +911,13 @@ def run_scp_server(sftp_server, command, stdin, stdout, stderr):
     try:
         args = _SCPArgParser().parse(command)
     except ValueError as exc:
+        stdin.logger.info('Error starting SCP server: %s', str(exc))
         stderr.write(b'scp: ' + str(exc).encode('utf-8') + b'\n')
         stderr.channel.exit(1)
         return
 
+    stdin.logger.info('Starting SCP server, args: %s', command[4:].strip())
+
     fs = SFTPServerFile(sftp_server)
 
     if args.source:
