@@ -57,6 +57,12 @@ class SSHReader:
 
         return self._chan
 
+    @property
+    def logger(self):
+        """The SSH logger associated with this stream"""
+
+        return self._chan.logger
+
     def get_extra_info(self, name, default=None):
         """Return additional information about this stream
 
@@ -73,19 +79,19 @@ class SSHReader:
     def read(self, n=-1):
         """Read data from the stream
 
-           This method is a coroutine which reads up to ``n`` bytes
-           or characters from the stream. If ``n`` is not provided or
-           set to ``-1``, it reads until EOF or a signal is received.
+           This method is a coroutine which reads up to `n` bytes
+           or characters from the stream. If `n` is not provided or
+           set to `-1`, it reads until EOF or a signal is received.
 
            If EOF is received and the receive buffer is empty, an
-           empty bytes or str object is returned.
+           empty `bytes` or `str` object is returned.
 
            If the next data in the stream is a signal, the signal is
            delivered as a raised exception.
 
-           .. note:: Unlike traditional ``asyncio`` stream readers,
-                     the data will be delivered as either bytes or
-                     a str depending on whether an encoding was
+           .. note:: Unlike traditional `asyncio` stream readers,
+                     the data will be delivered as either `bytes` or
+                     a `str` depending on whether an encoding was
                      specified when the underlying channel was opened.
 
         """
@@ -97,11 +103,11 @@ class SSHReader:
         """Read one line from the stream
 
            This method is a coroutine which reads one line, ending in
-           ``'\\n'``.
+           `'\\n'`.
 
-           If EOF is received before ``'\\n'`` is found, the partial
+           If EOF is received before `'\\n'` is found, the partial
            line is returned. If EOF is received and the receive buffer
-           is empty, an empty bytes or str object is returned.
+           is empty, an empty `bytes` or `str` object is returned.
 
            If the next data in the stream is a signal, the signal is
            delivered as a raised exception.
@@ -119,7 +125,7 @@ class SSHReader:
 
     @asyncio.coroutine
     def readuntil(self, separator):
-        """Read data from the stream until ``separator`` is seen
+        """Read data from the stream until `separator` is seen
 
            This method is a coroutine which reads from the stream until
            the requested separator is seen. If a match is found, the
@@ -127,7 +133,7 @@ class SSHReader:
 
            If EOF or a signal is received before a match occurs, an
            :exc:`IncompleteReadError <asyncio.IncompleteReadError>`
-           is raised and its ``partial`` attribute will contain the
+           is raised and its `partial` attribute will contain the
            data in the stream prior to the EOF or signal.
 
            If the next data in the stream is a signal, the signal is
@@ -144,9 +150,9 @@ class SSHReader:
            This method is a coroutine which reads exactly n bytes or
            characters from the stream.
 
-           If EOF or a signal is received in the stream before ``n``
+           If EOF or a signal is received in the stream before `n`
            bytes are read, an :exc:`IncompleteReadError
-           <asyncio.IncompleteReadError>` is raised and its ``partial``
+           <asyncio.IncompleteReadError>` is raised and its `partial`
            attribute will contain the data before the EOF or signal.
 
            If the next data in the stream is a signal, the signal is
@@ -159,7 +165,7 @@ class SSHReader:
     def at_eof(self):
         """Return whether the stream is at EOF
 
-           This method returns ``True`` when EOF has been received and
+           This method returns `True` when EOF has been received and
            all data in the stream has been read.
 
         """
@@ -186,6 +192,12 @@ class SSHWriter:
 
         return self._chan
 
+    @property
+    def logger(self):
+        """The SSH logger associated with this stream"""
+
+        return self._chan.logger
+
     def get_extra_info(self, name, default=None):
         """Return additional information about this stream
 
@@ -232,9 +244,9 @@ class SSHWriter:
 
            This method writes bytes or characters to the stream.
 
-           .. note:: Unlike traditional ``asyncio`` stream writers,
-                     the data must be supplied as either bytes or
-                     a str depending on whether an encoding was
+           .. note:: Unlike traditional `asyncio` stream writers,
+                     the data must be supplied as either `bytes` or
+                     a `str` depending on whether an encoding was
                      specified when the underlying channel was opened.
 
         """
@@ -552,6 +564,9 @@ class SSHServerStreamSession(SSHStreamSession, SSHServerSession):
     def exec_requested(self, command):
         """Return whether execution of a command can be requested"""
 
+        # Avoid incorrect pylint suggestion to use ternary
+        # pylint: disable=consider-using-ternary
+
         return ((self._allow_scp and command.startswith('scp ')) or
                 bool(self._session_factory))
 
@@ -588,7 +603,7 @@ class SSHServerStreamSession(SSHStreamSession, SSHServerSession):
             handler = self._session_factory(stdin, stdout, stderr)
 
         if asyncio.iscoroutine(handler):
-            self._conn.create_task(handler)
+            self._conn.create_task(handler, stdin.logger)
 
     def break_received(self, msec):
         """Handle an incoming break on the channel"""
@@ -603,10 +618,11 @@ class SSHServerStreamSession(SSHStreamSession, SSHServerSession):
         self._recv_buf[None].append(SignalReceived(signal))
         self._unblock_read(None)
 
-    def terminal_size_changed(self, *args):
+    def terminal_size_changed(self, width, height, pixwidth, pixheight):
         """Handle an incoming terminal size change on the channel"""
 
-        self._recv_buf[None].append(TerminalSizeChanged(*args))
+        self._recv_buf[None].append(TerminalSizeChanged(width, height,
+                                                        pixwidth, pixheight))
         self._unblock_read(None)
 
 
@@ -622,11 +638,13 @@ class SSHSocketStreamSession(SSHStreamSession):
         """Start a session for this newly opened socket channel"""
 
         if self._handler_factory:
-            handler = self._handler_factory(SSHReader(self, self._chan),
-                                            SSHWriter(self, self._chan))
+            reader = SSHReader(self, self._chan)
+            writer = SSHWriter(self, self._chan)
+
+            handler = self._handler_factory(reader, writer)
 
             if asyncio.iscoroutine(handler):
-                self._conn.create_task(handler)
+                self._conn.create_task(handler, reader.logger)
 
 
 class SSHTCPStreamSession(SSHSocketStreamSession, SSHTCPSession):
