@@ -1,6 +1,6 @@
-#!/usr/bin/env python3.4
+#!/usr/bin/env python3.5
 #
-# Copyright (c) 2013-2015 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2013-2016 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -14,17 +14,16 @@
 
 import asyncio, asyncssh, sys
 
-@asyncio.coroutine
-def run_client():
-    with (yield from asyncssh.connect('localhost')) as conn:
-        reader, writer = yield from conn.open_connection('www.google.com', 80)
+async def run_client():
+    async with asyncssh.connect('localhost') as conn:
+        reader, writer = await conn.open_connection('www.google.com', 80)
 
         # By default, TCP connections send and receive bytes
         writer.write(b'HEAD / HTTP/1.0\r\n\r\n')
         writer.write_eof()
 
         # We use sys.stdout.buffer here because we're writing bytes
-        response = yield from reader.read()
+        response = await reader.read()
         sys.stdout.buffer.write(response)
 
 try:
