@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2015 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2014-2016 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -11,6 +11,3 @@
 #     Ron Frederick - initial implementation, API, and documentation
 
 """Unit tests for AsyncSSH"""
-
-from . import test_asn1, test_cipher, test_compression, test_known_hosts
-from . import test_mac, test_public_key, test_saslprep, util
