@@ -143,11 +143,6 @@ class _ServerChannel(asyncssh.SSHServerChannel):
 
         asyncio.get_event_loop().call_later(0.1, self._report_response, True)
 
-    def send_packet(self, pkttype, *args):
-        """Send a packet for unit testing (bypassing state checks)"""
-
-        self._send_packet(pkttype, *args)
-
     @asyncio.coroutine
     def open_session(self):
         """Attempt to open a session on the client"""
@@ -385,7 +380,7 @@ class _ChannelServer(Server):
             stdin.channel.send_packet(MSG_CHANNEL_CLOSE)
             stdin.channel.exit(1)
         elif action == 'unexpected_auth':
-            self._conn.send_packet(Byte(MSG_USERAUTH_REQUEST), String('guest'),
+            self._conn.send_packet(MSG_USERAUTH_REQUEST, String('guest'),
                                    String('ssh-connection'), String('none'))
         elif action == 'invalid_response':
             stdin.channel.send_packet(MSG_CHANNEL_SUCCESS)
@@ -395,10 +390,19 @@ class _ChannelServer(Server):
         stdin.channel.close()
         yield from stdin.channel.wait_closed()
 
+    @asyncio.coroutine
+    def _conn_close(self):
+        """Close the connection during a channel open"""
+
+        self._conn.close()
+        yield from asyncio.sleep(0.1)
+        return _EchoServerSession()
+
     def begin_auth(self, username):
         """Handle client authentication request"""
 
-        return username not in {'guest', 'conn_close', 'close', 'echo',
+        return username not in {'guest', 'conn_close_startup',
+                                'conn_close_open', 'close', 'echo',
                                 'no_channels', 'no_pty', 'task_error'}
 
     def session_requested(self):
@@ -409,9 +413,11 @@ class _ChannelServer(Server):
         with patch('asyncssh.connection.SSHServerChannel', _ServerChannel):
             channel = self._conn.create_server_channel()
 
-            if username == 'conn_close':
+            if username == 'conn_close_startup':
                 self._conn.close()
                 return False
+            elif username == 'conn_close_open':
+                return (channel, self._conn_close())
             elif username in {'close', 'echo', 'no_pty', 'task_error'}:
                 return (channel, _EchoServerSession())
             elif username != 'no_channels':
@@ -589,7 +595,17 @@ class _TestChannel(ServerTestCase):
     def test_conn_close_during_startup(self):
         """Test connection close during channel startup"""
 
-        with (yield from self.connect(username='conn_close')) as conn:
+        with (yield from self.connect(username='conn_close_startup')) as conn:
+            with self.assertRaises(asyncssh.ChannelOpenError):
+                yield from _create_session(conn)
+
+        yield from conn.wait_closed()
+
+    @asynctest
+    def test_conn_close_during_open(self):
+        """Test connection close during channel open"""
+
+        with (yield from self.connect(username='conn_close_open')) as conn:
             with self.assertRaises(asyncssh.ChannelOpenError):
                 yield from _create_session(conn)
 
@@ -663,6 +679,18 @@ class _TestChannel(ServerTestCase):
 
         yield from conn.wait_closed()
 
+    @asynctest
+    def test_unknown_channel_request(self):
+        """Test sending unknown channel request"""
+
+        with patch('asyncssh.connection.SSHClientChannel', _ClientChannel):
+            with (yield from self.connect()) as conn:
+                chan, _ = yield from _create_session(conn)
+
+                self.assertFalse((yield from chan.make_request('unknown')))
+
+            yield from conn.wait_closed()
+
     @asynctest
     def test_invalid_channel_request(self):
         """Test sending non-ASCII channel request"""
@@ -1191,6 +1219,20 @@ class _TestChannel(ServerTestCase):
 
         yield from conn.wait_closed()
 
+    @asynctest
+    def test_full_terminal_size_change(self):
+        """Test sending full terminal size change"""
+
+        with (yield from self.connect()) as conn:
+            chan, session = yield from _create_session(conn, 'signals',
+                                                       term_type='ansi')
+
+            chan.change_terminal_size(80, 24, 480, 240)
+            yield from chan.wait_closed()
+            self.assertEqual(session.exit_signal_msg, '(80, 24, 480, 240)')
+
+        yield from conn.wait_closed()
+
     @asynctest
     def test_exit_status(self):
         """Test receiving exit status"""
