@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2017 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2016-2018 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -108,6 +108,12 @@ class _PublicKeyServer(Server):
 class _AsyncPublicKeyServer(_PublicKeyServer):
     """Server for testing async public key authentication"""
 
+    @asyncio.coroutine
+    def begin_auth(self, username):
+        """Handle client authentication request"""
+
+        return super().begin_auth(username)
+
     @asyncio.coroutine
     def validate_public_key(self, username, key):
         """Return whether key is an authorized client key for this user"""
@@ -215,7 +221,7 @@ class _KbdintClient(asyncssh.SSHClient):
 
         # pylint: disable=unused-argument
 
-        if len(prompts) == 0:
+        if not prompts:
             return []
         elif self._responses:
             result = self._responses
