@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2017 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2015-2018 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -66,7 +66,7 @@ class _KexConnectionStub(ConnectionStub):
 
         packet = SSHPacket(data)
         pkttype = packet.get_byte()
-        self._kex.process_packet(pkttype, packet)
+        self._kex.process_packet(pkttype, None, packet)
 
     def get_hash_prefix(self):
         """Return the bytes used in calculating unique connection hashes"""
