@@ -1,4 +1,4 @@
-# Copyright (c) 2016 by Ron Frederick <ronf@timeheart.net>.
+# Copyright (c) 2016-2018 by Ron Frederick <ronf@timeheart.net>.
 # All rights reserved.
 #
 # This program and the accompanying materials are made available under
@@ -24,12 +24,10 @@ import asyncssh
 from .server import ServerTestCase
 from .util import asynctest, echo
 
-
+@asyncio.coroutine
 def _handle_client(process):
     """Handle a new client request"""
 
-    # pylint: disable=no-self-use
-
     action = process.command or process.subsystem
     if not action:
         action = 'echo'
